package go_swagger

import (
	"fmt"
	"path/filepath"
	"strings"

	"github.com/juju/errors"

	"git.home.m-and-m.ovh/mderasse/gouick/helpers"
	"git.home.m-and-m.ovh/mderasse/gouick/models"

	log "github.com/sirupsen/logrus"
)

type launcherProcess struct {
	Binary string
	Name   string
}

type launcherTemplate struct {
	APIs    []*launcherProcess
	Workers []*launcherProcess
}

// GenerateLauncher will generate a launcher.sh based on the files found in cmd directory.
func (a APIType) GenerateLauncher(path string, config *models.Config) error {
	log.Debugf("Starting %s - %s", a.GetName(), helpers.GetCurrentFuncName())

	cmdPath := filepath.Join(path, "cmd")
	templatePath := filepath.Join(templateDirectory, "launcher.sh.tmpl")
	savePath := filepath.Join(path, "launcher.sh")

	data := launcherTemplate{
		APIs:    []*launcherProcess{},
		Workers: []*launcherProcess{},
	}

	cmdDirectories, err := filepath.Glob(filepath.Join(cmdPath, "*"))
	if err != nil {
		log.Error("Fail to list cmd directory")
		return errors.Trace(err)
	}

	for _, absDir := range cmdDirectories {
		dir := strings.TrimPrefix(absDir, fmt.Sprintf("%s%c", cmdPath, filepath.Separator))

		// if we have a suffix -server or -api we are on an API.
		// if we have a prefix worker- we are on a worker.
		if strings.HasSuffix(dir, "-server") || strings.HasSuffix(dir, "-api") {
			data.APIs = append(data.APIs, &launcherProcess{
				Binary: dir,
				Name:   dir,
			})
		} else if strings.HasPrefix(dir, "worker-") {
			data.APIs = append(data.Workers, &launcherProcess{
				Binary: dir,
				Name:   strings.TrimPrefix(dir, "worker-"),
			})
		}
	}

	err = helpers.WriteTemplate(templatePath, savePath, data)
	if err != nil {
		return errors.Trace(err)
	}

	return nil
}
